package com.momo

import com.example.onrampwebviewsdk.OnrampSDK
import com.example.onrampwebviewsdk.listeners.OnrampSDKEventListener
import com.example.onrampwebviewsdk.ui.WidgetResponse
import com.facebook.react.bridge.Arguments
import com.facebook.react.bridge.ReactApplicationContext
import com.facebook.react.bridge.ReactContextBaseJavaModule
import com.facebook.react.bridge.ReactMethod
import com.facebook.react.bridge.ReadableMap
import com.facebook.react.bridge.UiThreadUtil.runOnUiThread
import com.facebook.react.bridge.WritableMap
import com.facebook.react.bridge.WritableNativeMap
import com.facebook.react.modules.core.DeviceEventManagerModule
import com.google.gson.Gson
import java.util.Objects
import org.json.JSONException
import org.json.JSONObject

class MomoModule(private val reactContext: ReactApplicationContext) :
        ReactContextBaseJavaModule(reactContext), OnrampSDKEventListener {

    override fun getName(): String {
        return NAME
    }

    companion object {
        const val NAME = "Momo"
    }

    private fun sendEvent(eventName: String, params: WritableMap?) {
        reactContext
                .getJSModule(DeviceEventManagerModule.RCTDeviceEventEmitter::class.java)
                .emit(eventName, params)
    }

    @ReactMethod
    fun startSdk(args: ReadableMap) {
        var appId = 0
        var walletAddress: String? = null
        var flowType = 1
        var coinCode: String? = null
        var network: String? = null
        var coinAmount: Double? = null
        var fiatAmount: Double? = null
        var merchantRecognitionId: String? = null
        var paymentMethod: Int? = null
        var authToken: String? = null
        var assetDescription: String? = null
        var assetImage: String? = null
        var paymentAddress: String? = null

        if (args.hasKey("appId")) {
            appId = args.getInt("appId")
        }
        if (args.hasKey("walletAddress")) {
            walletAddress = args.getString("walletAddress").toString()
        }
        if (args.hasKey("flowType")) {
            flowType = args.getInt("flowType")
        }
        if (args.hasKey("coinCode")) {
            coinCode = args.getString("coinCode")
        }
        if (args.hasKey("network")) {
            network = args.getString("network")
        }
        if (args.hasKey("coinAmount")) {
            coinAmount = args.getDouble("coinAmount")
        }
        if (args.hasKey("fiatAmount")) {
            fiatAmount = args.getDouble("fiatAmount")
        }
        if (args.hasKey("merchantRecognitionId")) {
            merchantRecognitionId = args.getString("merchantRecognitionId")
        }
        if (args.hasKey("paymentMethod")) {
            paymentMethod = args.getInt("paymentMethod")
        }
        if (args.hasKey("authToken")) {
            authToken = args.getString("authToken")
        }
        if (args.hasKey("assetDescription")) {
            assetDescription = args.getString("assetDescription")
        }
        if (args.hasKey("assetImage")) {
            assetImage = args.getString("assetImage")
        }
        if (args.hasKey("paymentAddress")) {
            paymentAddress = args.getString("paymentAddress")
        }

        OnrampSDK.startOnrampSDK(
                reactApplicationContext,
                appId,
                walletAddress!!,
                flowType,
                coinCode,
                network,
                coinAmount,
                fiatAmount,
                merchantRecognitionId,
                paymentMethod,
                authToken,
                assetDescription,
                assetImage,
                paymentAddress
        )

        OnrampSDK.registerOnrampSDKEventListener(this)
    }

    @Throws(JSONException::class)
    private fun convertJsonToMap(jsonObject: JSONObject): WritableMap? {
        val map: WritableMap = WritableNativeMap()
        val iterator = jsonObject.keys()
        while (iterator.hasNext()) {
            val key = iterator.next()
            when (val value = jsonObject[key]) {
                is JSONObject -> {
                    map.putMap(key, convertJsonToMap(value))
                }
                is Boolean -> {
                    map.putBoolean(key, value)
                }
                is Int -> {
                    map.putInt(key, value)
                }
                is Double -> {
                    map.putDouble(key, value)
                }
                is String -> {
                    map.putString(key, value)
                }
                else -> {
                    map.putString(key, value.toString())
                }
            }
        }
        return map
    }

    @ReactMethod
    override fun onDataReceived(data: WidgetResponse) {
        val resultMap = Arguments.createMap()
        val g = Gson()
        resultMap.putString("type", Objects.requireNonNull(data).type)
        resultMap.putBoolean("isOnramp", Objects.requireNonNull(data).isOnramp)

        val jo = JSONObject(g.toJson(data.data))
        val wm = convertJsonToMap(jo)
        resultMap.putMap("data", wm)

        when (data.type) {
            "ONRAMP_WIDGET_CLOSE_REQUEST" -> {
                runOnUiThread { OnrampSDK.stopOnrampSDK() }
            }
        }

        sendEvent("widgetEvents", resultMap)
    }

    @ReactMethod
    fun addListener(eventName: String) {
        // Keep: Required for RN built in Event Emitter Calls.
    }

    @ReactMethod
    fun removeListeners(count: Int) {
        // Keep: Required for RN built in Event Emitter Calls.
    }
}
